// code written by Bart Huyskens  05/01/2018  for www.e2cre8.be
// hardware: Brainbox Robot (www.e2cre8.be >> Brainbox Robot)(Based on Arduino Leonardo) ; HC06 Bluetooth module www.e2cre8.be ; Android Smartphone ; I2C LCD 4x20 www.e2cre8.be
// software: Arduino IDE 1.6.11 ; Appinventor
// Principle of communication:
// The app sends data to the uC in this format: "L:xxx;R:xxx;" where xxx is data between 0-255. 
// The speed is determined by the TimerInterval of Timer1 in Appinventor and this needs to be identical to the variable "TimerIntervalAppinventor" in this program
// Speeds up to 100msec seem to be OK, but if you do not need this speed, use 1000msec
// The uC waits checks for incoming data at half of "TimerIntervalAppinventor" -time to be sure to not miss to much
// The uC send 4 sensor values to the app, every 2x "TimerIntervalAppinventor". This needs to be double the time for appinventor to be ready to receive the next data
// comment out the LCD lines if you do not have a I2C LCD connected

// Arduino preprocessor directives //
#include <Wire.h>
#include <LCD.h>
#include <LiquidCrystal_I2C.h>

#define TimerIntervalAppinventor 100 // this value needs to be equal to the value of the timer in Appinventor
                                    // tested & works pretty stable at 100msec, but if you don't need the speed, set it at 1000msec both here and in Appinventor
#define PWM_LEFTMOTOR 6             // pin number for speed left motor
#define DIR_LEFTMOTOR 14            // pin number for direction (forward / backward) left motor
#define PWM_RIGHTMOTOR 5            // pin number for speed right motor
#define DIR_RIGHTMOTOR 16           // pin number for direction (forward / backward) right motor

#define tx 1                        // pin number for TX pin
#define rx 0                        // pin number for RX pin
#define LIGHTSENSL A0   
#define LIGHTSENSR A1  
#define IRSENSL A2   
#define IRSENSR A3   
#define IRLEDL 9   
#define IRLEDR 10  
#define BUZZER 7 
#define LED11 11
#define LED13 13
#define LED30 30
#define LED17 17
#define LINEL 4
#define LINER 12
#define SWL 8
#define SWR 22

// Declaration of variables //

int NrOfCycles;       // to slow down the transmission of data
int YValue = 127;
int XValue = 127;
int YValueOld = 127;
int XValueOld = 127;
int i=0;
int z=0;
int connectionOK = 0;
int DC_LEFT = 0;
byte DIR_LEFT = 0;
int DC_RIGHT = 0;
byte DIR_RIGHT = 0;
byte ReceiveX = 127;
byte ReceiveY = 127;
int DISTSENSL = 0;
int DISTSENSR = 0;
int LIGHTSENSVALUEL = 0;
int LIGHTSENSVALUER = 0;

char inSerial[25]="L:0;R:0;Z";    // Array of characters (is not a string)- used to read the received data
String myString;                  // used to do string manipulations on the received data    

LiquidCrystal_I2C lcd_I2C_27(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);

void setup()
  {
  Serial.begin(9600);             //Bautrate speed For Serial monitor 
  Serial1.begin(9600);            //Bautrate speed For comms with HC06 - baudrate 9600

// configuration of input/output pins 
  pinMode(DIR_LEFTMOTOR, OUTPUT);
  pinMode(DIR_RIGHTMOTOR, OUTPUT);
  pinMode(PWM_LEFTMOTOR, OUTPUT);
  pinMode(PWM_RIGHTMOTOR, OUTPUT);
  pinMode(tx, OUTPUT);
  pinMode(rx, INPUT);
  pinMode(IRLEDL, OUTPUT);
  pinMode(IRLEDR, OUTPUT);
  pinMode(BUZZER, OUTPUT);
  pinMode(LED11, OUTPUT);
  pinMode(LED13, OUTPUT);
  pinMode(LED30, OUTPUT);
  pinMode(LED17, OUTPUT);
  pinMode(LINEL, INPUT);
  pinMode(LINER, INPUT);
  pinMode(SWL, INPUT);
  pinMode(SWR, INPUT);
    
  lcd_I2C_27.begin (20, 4);
  lcd_I2C_27.setBacklight(HIGH);
  }

void loop()
{
 delay(TimerIntervalAppinventor/2);       // timerInterval of Appinventor devided by 2, to be ready to receive all incoming data
 NrOfCycles = NrOfCycles + 1;             // transmission of data can only execute after 4 receive cycles

 //***RECEIVING DATA ***//
 if (Serial1.available() > 0)             // If receiving serial data from HC06
    {
    connectionOK = 1;                    // only start sending data after reception of first data -else uC crashes
    i=0;
    while (Serial1.available())         // while receiving serial data from HC06
      {
      inSerial[i]=Serial1.read();        // load incoming bytes in array
      i++;                              // increment i 
      }
    }

  //***TRANSMITTING DATA ****//
  if ((NrOfCycles >= 4)&& connectionOK)        // transmission of data can only execute after 4 receive cycles for Appinventor to be ready to receive data
  {
  Serial1.print(DISTSENSL);            // send varialble via Serial1 to HC06
  Serial1.print('|');                 // send '|' character - appinventor uses this to split the data
  Serial1.print(LIGHTSENSVALUEL);
  Serial1.print('|');
  Serial1.print(LIGHTSENSVALUER);
  Serial1.print('|');
  Serial1.print(DISTSENSR);
  Serial1.print('|');
  NrOfCycles = 0;
  }

  // Decode incoming data (ascii format) to usable values
    Serial.print(inSerial);               // Serial Monitor print received data 
    myString = String(inSerial);          // convert array to string to be able to use string manipulation instructions (indexOf...)

    int Xpos = myString.indexOf('X');                           // find the position of character 'R' in the string
    int m = 0;
    XValueOld = XValue;
    XValue = 0; // reset L value
    while ((myString.charAt(Xpos+2+m)) != ';')                  //convert characters into RValue numbers untill character ';'
      {                                                         // start 2 positions after character 'R'
      XValue = XValue *10 + ((myString.charAt(Xpos+2+m))-48);   //char '0' has ascii value 48
      if ((XValue > 255) || (XValue <0)) XValue = XValueOld;    // protection from corrupt data
      m++;
      }
    
    int Ypos = myString.indexOf('Y');                           // find the position of character 'R' in the string
    m = 0;
    YValueOld = YValue;
    YValue = 0;                                                 // Reset R Value
    while ((myString.charAt(Ypos+2+m)) != ';')                  //convert characters into RValue numbers untill character ';'
      {                                                         // start 2 positions after character 'R'
      YValue = YValue *10 + ((myString.charAt(Ypos+2+m))-48);   //char '0' has ascii value 48
      if ((YValue > 255) || (YValue <0)) YValue = YValueOld;    // protection from corrupt data
      m++;
      }


    Serial.print("\t");                   // Serial monitor - TAB
    Serial.println(XValue);               // Serial monitor Value of R
    Serial.print("\t");                   // Serial monitor - TAB
    Serial.println(YValue);               // Serial monitor Value of L
  
  lcd_I2C_27.clear(); 
  lcd_I2C_27.setCursor( (1) - 1, (1) - 1 );
  lcd_I2C_27.print( "WWW.E2CRE8.BE BT" );   
   
  lcd_I2C_27.setCursor( (1) - 1, (2) - 1 );
  lcd_I2C_27.print( "L:" );
  lcd_I2C_27.print( String(ReceiveX));
  lcd_I2C_27.setCursor( (9) - 1, (2) - 1 );
  lcd_I2C_27.print( "R:" );
  lcd_I2C_27.print( String(ReceiveY));
  
  if (connectionOK != 1)
{
  lcd_I2C_27.setCursor( (1) - 1, (3) - 1 );
  lcd_I2C_27.print( "NO CONNECT. WITH APP" ); 
}
else
{
  lcd_I2C_27.setCursor( (1) - 1, (3) - 1 );
  lcd_I2C_27.print( "CONNECTED WITH APP" ); 
}

///****DRIVING MOTORS****///
// recalculating Left-right values from X/Y values - 4 quadrants

ReceiveY = YValue;      // copy received value to workvariable
ReceiveX = XValue;
  if ((ReceiveY >=128)&&(ReceiveX >= 128)) // forward - right
    {
    DIR_LEFT = 0; // forward
    DIR_RIGHT = 0; // forward
    DC_LEFT = ((ReceiveY - 127)*2);
    if (DC_LEFT > 254)DC_LEFT = 255;
    if (DC_LEFT < 0)DC_LEFT = 0;
    DC_RIGHT = ((ReceiveY - 127)*2)-(ReceiveX - 127);
    if (DC_RIGHT > 254)DC_RIGHT = 255;
    if (DC_RIGHT < 0)DC_RIGHT = 0;
    }
    
  else if ((ReceiveY >= 128)&&(ReceiveX <= 127)) // forward left
    {
    DIR_LEFT = 0; // forward
    DIR_RIGHT = 0; // forward
    DC_LEFT = ((ReceiveY - 127)*2)-(127-ReceiveX);
    if (DC_LEFT > 254)DC_LEFT = 255;
    if (DC_LEFT < 0)DC_LEFT = 0;
    DC_RIGHT = ((ReceiveY - 127)*2);
    if (DC_RIGHT > 254)DC_RIGHT = 255;
    if (DC_RIGHT < 0)DC_RIGHT = 0;
    }
    
   else if ((ReceiveY <= 127)&&(ReceiveX <= 127)) // forward left
    {
    DIR_LEFT = 1; // backwards
    DIR_RIGHT = 1; // backwards
    DC_LEFT = 255-(((127-ReceiveY)*2)-(127-ReceiveX));
    if (DC_LEFT > 254)DC_LEFT = 255;
    if (DC_LEFT < 0)DC_LEFT = 0;
    DC_RIGHT = 255-((127-ReceiveY)*2);
    if (DC_RIGHT > 254)DC_RIGHT = 255;
    if (DC_RIGHT < 0)DC_RIGHT = 0;
    }
    else if ((ReceiveY <= 127)&&(ReceiveX >= 128)) // forward left
    {
    DIR_LEFT = 1; // backwards
    DIR_RIGHT = 1; // backwards
    DC_LEFT = 255-((127-ReceiveY)*2);
    if (DC_LEFT > 254)DC_LEFT = 255;
    if (DC_LEFT < 0)DC_LEFT = 0;
    DC_RIGHT = 255-(((127-ReceiveY)*2)-(ReceiveX-127));
    if (DC_RIGHT > 254)DC_RIGHT = 255;
    if (DC_RIGHT < 0)DC_RIGHT = 0;
    }
    else
    {
    DIR_LEFT = 0; // forward
    DIR_RIGHT = 0; // forward
    DC_LEFT =0;
    DC_RIGHT =0;
    }
    
    // these lines write the calculated values to the two motors of the BBR
    analogWrite(PWM_LEFTMOTOR, DC_LEFT);
    digitalWrite(DIR_LEFTMOTOR, DIR_LEFT);
    analogWrite(PWM_RIGHTMOTOR, DC_RIGHT);
    digitalWrite(DIR_RIGHTMOTOR, DIR_RIGHT);
 
 
 // READING SENSOR VALUES
    digitalWrite( IRLEDL , HIGH );              // IR emittor ON
    delay( 1 );                                 // wait 1msec to stabilize
    DISTSENSL = analogRead(IRSENSL) ;             // read IR sensor
    digitalWrite( IRLEDL , LOW );               // IR emittor OFF
    digitalWrite( IRLEDR , HIGH );              // IR emittor ON
    delay( 1 );                                 // wait 1msec to stabilize
    DISTSENSR = analogRead(IRSENSR) ;             // read IR sensor  
    digitalWrite( IRLEDR , LOW );               // IR emittor OFF

    LIGHTSENSVALUEL = analogRead(LIGHTSENSL) ;    // read lightsensor value
    LIGHTSENSVALUER = analogRead(LIGHTSENSR) ;    // read lightsensor value

 
    
}

  



